import axios from "axios"
import {ApiCallID} from "./ApiCallID";

/**
 * Base Construct for every Call
 */
export abstract class AbstractCall<RequestPayload extends BaseRequestPayload, ResponsePayload extends BaseResponsePayload>{

    //Request Data
    readonly data : Request<RequestPayload>;

    public constructor(data : Request<RequestPayload>){
        this.data = data;
    }

    abstract getRequestName() : string;

    /**
     * Method that sends the data
     * @param {string} ip optional Standard is api.meetznow.io
     * @returns {Promise<Response<ResponsePayload extends BaseResponsePayload>>}
     */
    async get(ip? : string) : Promise<Response<ResponsePayload>> {

        let IP = "";

        if(ip) IP  = ip;
        else if(process && process.env && process.env.MEETZNOW_BACKEND) IP = process.env.MEETZNOW_BACKEND;
        else if(process && process.env && process.env.VUE_APP_MEETZNOW_BACKEND_IP) IP = process.env.VUE_APP_MEETZNOW_BACKEND_IP;
        else throw new Error("No backend IP provided! Either pass a IP to .get(ip? : string) or set the environment variable MEETZNOW_BACKEND or VUE_APP_MEETZNOW_BACKEND_IP.");

        const url = `http${IP === "localhost:3000" ? "" : "s"}://${IP}/${this.getRequestName()}`;

        const responseAxios = await axios.post(url, this.data);

        if(responseAxios.status !== 200 || !responseAxios.data)
            throw new Error(JSON.stringify(responseAxios));

        const response = responseAxios.data.valueOf() as Response<ResponsePayload>;

        if(response.error)
            throw new Error(response.error.message);

        return response;
    }
}

/**
 * Base Request Structure
 */
export interface Request<RequestPayload extends BaseRequestPayload>{
    token?:any,
    payload : RequestPayload
}

/**
 * Base Response Structure
 */
export interface Response<ResponsePayload extends BaseResponsePayload>{
    id : ApiCallID
    successful : boolean
    error? : {
        message : string
        code? : string
    }
    payload? : ResponsePayload
}

export interface BaseRequestPayload{}

export interface BaseResponsePayload {}